import{aM as g,af as y,ay as O,y as E,an as V,V as F,R as W,aN as x,v as m,M as j,aO as B,x as f,H,b as L,e as M,h as T,k as u,g as _,aP as q,aQ as U,j as $,al as G,K as k,J as z,F as J,_ as K,aR as Q}from"./utils.Bch77dSh.js";function Y(e){var r=O,t=E;g(null),y(null);try{return e()}finally{g(r),y(t)}}const X=new Set,Z=new Set;function ee(e,r,t,n){function o(a){if(n.capture||re.call(r,a),!a.cancelBubble)return Y(()=>t.call(this,a))}return e.startsWith("pointer")||e.startsWith("touch")||e==="wheel"?W(()=>{r.addEventListener(e,o,n)}):r.addEventListener(e,o,n),o}function se(e,r,t,n,o){var a={capture:n,passive:o},i=ee(e,r,t,a);(r===document.body||r===window||r===document)&&x(()=>{r.removeEventListener(e,i,a)})}function ue(e){for(var r=0;r<e.length;r++)X.add(e[r]);for(var t of Z)t(e)}function re(e){var A;var r=this,t=r.ownerDocument,n=e.type,o=((A=e.composedPath)==null?void 0:A.call(e))||[],a=o[0]||e.target,i=0,d=e.__root;if(d){var c=o.indexOf(d);if(c!==-1&&(r===document||r===window)){e.__root=r;return}var S=o.indexOf(r);if(S===-1)return;c<=S&&(i=c)}if(a=o[i]||e.target,a!==r){V(e,"currentTarget",{configurable:!0,get(){return a||t}});var R=O,D=E;g(null),y(null);try{for(var p,N=[];a!==null;){var w=a.assignedSlot||a.parentNode||a.host||null;try{var v=a["__"+n];if(v!==void 0&&!a.disabled)if(F(v)){var[P,...C]=v;P.apply(a,[e,...C])}else v.call(a,e)}catch(h){p?N.push(h):p=h}if(e.cancelBubble||w===r||w===null)break;a=w}if(p){for(let h of N)queueMicrotask(()=>{throw h});throw p}}finally{e.__root=r,delete e.currentTarget,g(R),y(D)}}}let s;function ce(){s=void 0}function le(e){let r=null,t=f;var n;if(f){for(r=u,s===void 0&&(s=_(document.head));s!==null&&(s.nodeType!==8||s.data!==H);)s=L(s);s===null?M(!1):s=T(L(s))}f||(n=document.head.appendChild(m()));try{j(()=>e(n),B)}finally{t&&(M(!0),s=u,T(r))}}function I(e){var r=document.createElement("template");return r.innerHTML=e,r.content}function l(e,r){var t=E;t.nodes_start===null&&(t.nodes_start=e,t.nodes_end=r)}function fe(e,r){var t=(r&q)!==0,n=(r&U)!==0,o,a=!e.startsWith("<!>");return()=>{if(f)return l(u,null),u;o===void 0&&(o=I(a?e:"<!>"+e),t||(o=_(o)));var i=n?document.importNode(o,!0):o.cloneNode(!0);if(t){var d=_(i),c=i.lastChild;l(d,c)}else l(i,i);return i}}function de(e,r,t="svg"){var n=!e.startsWith("<!>"),o=`<${t}>${n?e:"<!>"+e}</${t}>`,a;return()=>{if(f)return l(u,null),u;if(!a){var i=I(o),d=_(i);a=_(d)}var c=a.cloneNode(!0);return l(c,c),c}}function _e(e=""){if(!f){var r=m(e+"");return l(r,r),r}var t=u;return t.nodeType!==3&&(t.before(t=m()),T(t)),l(t,t),t}function pe(){if(f)return l(u,null),u;var e=document.createDocumentFragment(),r=document.createComment(""),t=m();return e.append(r,t),l(r,t),e}function ve(e,r){if(f){E.nodes_end=u,$();return}e!==null&&e.before(r)}function he(e){return e.endsWith("capture")&&e!=="gotpointercapture"&&e!=="lostpointercapture"}const te=["beforeinput","click","change","dblclick","contextmenu","focusin","focusout","input","keydown","keyup","mousedown","mousemove","mouseout","mouseover","mouseup","pointerdown","pointermove","pointerout","pointerover","pointerup","touchend","touchmove","touchstart"];function be(e){return te.includes(e)}const ae={formnovalidate:"formNoValidate",ismap:"isMap",nomodule:"noModule",playsinline:"playsInline",readonly:"readOnly",srcobject:"srcObject"};function ge(e){return e=e.toLowerCase(),ae[e]??e}const ne=["touchstart","touchmove"];function ye(e){return ne.includes(e)}let b=!1;function me(e,r,t){const n=t[r]??(t[r]={store:null,source:G(void 0),unsubscribe:k});if(n.store!==e)if(n.unsubscribe(),n.store=e??null,e==null)n.source.v=void 0,n.unsubscribe=k;else{var o=!0;n.unsubscribe=z(e,a=>{o?n.source.v=a:K(n.source,a)}),o=!1}return J(n.source)}function Ee(){const e={};return x(()=>{for(var r in e)e[r].unsubscribe()}),e}function we(e){var r=b;try{return b=!1,[e(),b]}finally{b=r}}const oe="5";typeof window<"u"&&(window.__svelte||(window.__svelte={v:new Set})).v.add(oe);Q();export{ve as a,me as b,X as c,ce as d,l as e,we as f,pe as g,re as h,ye as i,_e as j,he as k,ee as l,ue as m,ge as n,be as o,le as p,de as q,Z as r,Ee as s,fe as t,se as u};
