.annotation.svelte-8g3qhd{display:flex;cursor:pointer;justify-content:space-between}.sample.svelte-8g3qhd .image{width:calc(var(--sample-width));height:calc(var(--sample-height));-o-object-fit:contain;object-fit:contain}
