import{s as h,a as l,t as u,b as v}from"../chunks/legacy.MjBvvGLm.js";import{p as _,f as x,t as $,a as d,s as b,c as o,r as p}from"../chunks/utils.Bch77dSh.js";import{s as m}from"../chunks/render.EBua_3s6.js";import{i as E}from"../chunks/lifecycle.DsUxRvcF.js";import{p as j}from"../chunks/stores.BcFlh_A8.js";var k=u("<h1> </h1> <p> </p>",1);function B(i,f){_(f,!1);const n=h(),s=()=>v(j,"$page",n);E();var a=k(),t=x(a),c=o(t,!0);p(t);var r=b(t,2),g=o(r,!0);p(r),$(()=>{var e;m(c,s().status),m(g,(e=s().error)==null?void 0:e.message)}),l(i,a),d()}export{B as component};
